// ss.ccp
// Written by D Ayala and R Guerrero
//
//  The 3 continuous space models used in Ayala et al (2012) Evolution, part of the inv2lafit program
//
// Written as different functions, necessary for the interaction with the amebsa routine (from Numerical Recipes)/
// 
//
#include <iostream>
using std::cout;
using std::endl;

#include <fstream>
using std::ifstream;
using std::ofstream;

#include <sstream>
using std::stringstream;
using std::istringstream;

#include <iomanip>
using std::setw;
using std::setprecision;

#include <string>
using std::string;

#include<math.h>

#include <algorithm>
using std::max;

#include<vector>
using std::vector;

#include <map>
using std::map;

#include <boost/math/distributions/normal.hpp> // for normal_distribution
using boost::math::normal; 

#include "amebsa.h"
#include "population.h"
#include "init_commons.h"


Doub ssone (VecDoub_I & vec ) {
    Doub ret; 
    Doub penalty; 
    double epsilon_one_run;
    double maxGens_one_run;
    
    SETRUNPARS
	
    Genotype g0= {vec[0],1,vec[1]};   
    double F= vec[2];  
    double sigma = vec[3];
	
	
	//	lower bounds for fitness
	if(g0.SS<0 || g0.SI<0 ||g0.II<0) return penalty* (1.0 + fabs(vec[0])+fabs(vec[1]));
    
    // upper bounds for fitness
	for(int i=0;i<3;++i){
		if(vec[i]>10000) return penalty * (1.0+fabs(vec[i]));
	}
	//  bounds for F
    if(F >1 || F < 0) return penalty * (1.0+fabs(F));
	
    // bounds for migration distance
	if(sigma > 100|| sigma < 0) return penalty * (1.0+fabs(sigma));
	
    int nPops;
    vector<double> ss ;
    vector<double> si ;
    vector<double> ii ; 
    vector<double> distances ; 
    
	INPUTDATA
    
    vector< Genotype > initfreq;
    vector< Genotype > W;
    initfreq.resize(nPops); 
    W.resize(nPops);
    
    for (int k=0; k< nPops ; ++k ) {          
		W.at(k)=g0;
	}
    
    vector<double> mig_prob; 
    INIT_MIGRATION
    INIT_FREQS
    vector <Population> cameroon;  
    FILL_POPULATIONS_AND_RUN
    vector<Genotype> one_run= simulate(cameroon, mig_prob, W, F, epsilon_one_run, maxGens_one_run);	
	CALC_LogL
    
}

Doub sstwo (VecDoub_I & vec ) {
	Doub ret; 
    Doub penalty; 
    double epsilon_one_run;
    double maxGens_one_run;
    
	SETRUNPARS
    
	Genotype g0= {vec[0],1,vec[1]};   
	Genotype g1= {vec[2],1,vec[3]};
    double F= vec[4]; 
    double sigma = vec[5];
    double borderZ= vec[6];
    
    
    // bounds for fitness
	for(int i=0;i<4;++i){
		if(vec[i]< 0)return penalty * (1.0+fabs(vec[i]));
		if(vec[i]>10000) return penalty * (1.0+fabs(vec[i]));
	}    
    
    // bounds for F
	if(F>1||F<0) return penalty * (1.0+fabs(F));
    
    // bounds for migration distance
	if(sigma<0.5 || sigma > 50) return penalty * (1.0+fabs(sigma));
    
	//bounds for region Borders
	if(borderZ  <= 30 || borderZ >= 80) return penalty ;
    
    
    int nPops;
    vector<double> ss ;
    vector<double> si ;
    vector<double> ii ; 
    vector<double> distances ; 
    
    INPUTDATA	
	
    vector< Genotype > initfreq;
    vector< Genotype > W;
    initfreq.resize(nPops); 
    W.resize(nPops);
    
	for (int k=0; k< (int)borderZ ; ++k ) {          
		W.at(k)=g0;
	}
	
	for (int k= (int)borderZ; k< nPops ; ++k ) {
		W.at(k)=g1;
	}
	
    vector<double> mig_prob; 
	INIT_MIGRATION
    INIT_FREQS
    vector <Population> cameroon;       
	FILL_POPULATIONS_AND_RUN	
	vector<Genotype> one_run= simulate(cameroon, mig_prob, W, F, epsilon_one_run, maxGens_one_run);	
    CALC_LogL
    
}

Doub ssthree (VecDoub_I & vec ) {
    Doub ret; 
    Doub penalty; 
    double epsilon_one_run;
    double maxGens_one_run;
    
	SETRUNPARS
    
    int borderSH; int borderHR;
	stringstream general;
	general <<"borders.txt"; 
	ifstream fin( general.str().c_str());
	string line;
    int intemp;
	getline( fin, line);
	istringstream iss(line);
	while(iss >> intemp){
		borderSH=intemp;
	}
	getline( fin, line);
	iss.clear();
	iss.str(line);
	while(iss >> intemp){
		borderHR=intemp;
	}
	fin.close();
    
    // These next lines establish the relationship between the vec vector passed from the SimAnneal routine and the actual parameters in our model
    Genotype g0= {1,vec[0],vec[1]};   
	Genotype g1= {1,vec[2],vec[3]};
	Genotype g2= {1,vec[4],vec[5]};
    double F = vec[6];  
    double sigma= vec[7];
    
    // bounds for fitness
	for(int i=0;i<6;++i){
		if(vec[i]< 0)return penalty * (1.0+fabs(vec[i]));
		if(vec[i]>10000) return penalty * (1.0+fabs(vec[i]));
	}    
    
	//  here are the bounds for F
    if(vec[6]>1||vec[6]<0) return penalty * (1.0+fabs(vec[6]));
	// bounds for migration distance
    if(vec[7]<0) return penalty * (1.0+fabs(vec[7]));
	
    int nPops;
    vector<double> ss ;
    vector<double> si ;
    vector<double> ii ; 
    vector<double> distances ; 
    
    INPUTDATA	
	
    vector< Genotype > initfreq;
    vector< Genotype > W;
    initfreq.resize(nPops); 
    W.resize(nPops);
    
	for (int k=0; k< borderSH ; ++k ) {          
		W.at(k)=g0;
	}
	
	for (int k= borderSH; k< borderHR ; ++k ) {
		W.at(k)=g1;
	}
	
	for (int k=borderHR; k< nPops ; ++k ) {
		W.at(k)=g2;
	}
    
    vector<double> mig_prob; 
	INIT_MIGRATION
    INIT_FREQS
    vector <Population> cameroon;       
	FILL_POPULATIONS_AND_RUN	
	vector<Genotype> one_run= simulate(cameroon, mig_prob, W, F, epsilon_one_run, maxGens_one_run);	
    CALC_LogL
	
}

Doub ss3_F0 (VecDoub_I & vec ) {
    Doub ret; 
    Doub penalty; 
    double epsilon_one_run;
    double maxGens_one_run;
    
	SETRUNPARS
    
    int borderSH; int borderHR;
	stringstream general;
	general <<"borders.txt"; 
	ifstream fin( general.str().c_str());
	string line;
    int intemp;
	getline( fin, line);
	istringstream iss(line);
	while(iss >> intemp){
		borderSH=intemp;
	}
	getline( fin, line);
	iss.clear();
	iss.str(line);
	while(iss >> intemp){
		borderHR=intemp;
	}
	fin.close();
    
    // These next lines establish the relationship between the vec vecotr passed from the SimAnneal routine and the actual parameters in our model
    Genotype g0= {1,vec[0],vec[1]};   
	Genotype g1= {1,vec[2],vec[3]};
	Genotype g2= {1,vec[4],vec[5]};
    double F = 0;  
    double sigma= vec[6];
    
    // bounds for fitness
	for(int i=0;i<6;++i){
		if(vec[i]< 0)return penalty * (1.0+fabs(vec[i]));
		if(vec[i]>10000) return penalty * (1.0+fabs(vec[i]));
	}    
    
    // bounds for fitness
	for(int i=0;i<6;++i){
		if(vec[i]< 0)return penalty * (1.0+fabs(vec[i]));
		if(vec[i]>10000) return penalty * (1.0+fabs(vec[i]));
	}    
   	// bounds for migration distance
    if(vec[6]<0) return penalty * (1.0+fabs(vec[7]));
    

    int nPops;
    vector<double> ss ;
    vector<double> si ;
    vector<double> ii ; 
    vector<double> distances ; 
    
    INPUTDATA	
	
    vector< Genotype > initfreq;
    vector< Genotype > W;
    initfreq.resize(nPops); 
    W.resize(nPops);
    
	for (int k=0; k< borderSH ; ++k ) {          
		W.at(k)=g0;
	}
	
	for (int k= borderSH; k< borderHR ; ++k ) {
		W.at(k)=g1;
	}
	
	for (int k=borderHR; k< nPops ; ++k ) {
		W.at(k)=g2;
	}
    
    vector<double> mig_prob; 
	INIT_MIGRATION
    INIT_FREQS
    vector <Population> cameroon;       
	FILL_POPULATIONS_AND_RUN	
	vector<Genotype> one_run= simulate(cameroon, mig_prob, W, F, epsilon_one_run, maxGens_one_run);	
    CALC_LogL
	
}

